﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Diagnostics;

namespace Increment
{
    class Increment
    {
        private int count = 0;
        // private volatile int count = 0;  // won't prevent losing updates
        private object countLock = new object();    // can be used to ensure mutual exclusion on count

        private const int SLEEP_MS = 1;
        private int iterations = 0;

        public Increment(int workers, int iterations)
        {
            this.iterations = iterations;

            List<Thread> threads = new List<Thread>();
            for (int i = 0; i < workers; i++)
                threads.Add(new Thread(new ThreadStart(worker)));
            Stopwatch timer = new Stopwatch();
            timer.Start();
            foreach (Thread t in threads)
                t.Start();

            System.Console.WriteLine("Total threads = {0}", threads.Count);

            foreach (Thread t in threads)
                t.Join();
            timer.Stop();

            System.Console.WriteLine("Final count = {0}", count);
            System.Console.WriteLine("Elapsed ms = {0}", timer.ElapsedMilliseconds);
        }

        public void worker()
        {
            for (int i = 0; i < iterations; i++)
            {
                //lock (count)      // won't work since not an object type
                //lock (this)       // will work, lock on whole Increment object instance
                //lock (countLock)  // also works, use an object that exists just to serve as lock
                {
                    count++;    // read and write, so not atomic!
                    //Interlocked.Increment(ref count);  // this will work without need a lock
                }
                //Thread.Sleep(SLEEP_MS);
            }
        }

        static void Main(string[] args)
        {
            Increment increment = new Increment(int.Parse(args[0]), int.Parse(args[1]));
        }
    }
}
